<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Factory;

class TaskOptions {
    /** @var  EntityManager */
	private $entityManager;
    /** @var  Utilities */
	private $environment;
    /** @var  \Zeedhi\DevOps\Model\Projectmanagement\Repositories\TaskStatus */
    private $taskStatusRepository;
    /** @var  Factory\TaskOptions */
    private $taskOptionsFactory;

    public function __construct(EntityManager $entityManager, Utilities $environment, Factory\TaskOptions $taskOptionsFactory) {
		$this->entityManager 	= $entityManager;
		$this->environment 		= $environment;
		$this->taskOptionsFactory	= $taskOptionsFactory;
		$this->taskStatusRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskStatus');
	}

    /**
     * Gets registered task status from database.
     * @param $filters
     * @return mixed
     */
	public function findTaskStatus($filters){
		$organizationId = $this->environment->getCurrentOrganizationId();
        $filtersString = $this->environment->createQueryFiltered($filters);
		$taskStatus 	= $this->taskStatusRepository->findTaskStatus($organizationId,$filtersString);
		$taskStatus 	= Utilities::setEssentialValuesToDataset($taskStatus);
		return $taskStatus;
	}

    /**
     * Creates a new task status.
     * @param $control
     * @param $label
     * @param $isIssue
     * @param $taskTypeId
     * @param $isInitial
     * @param null $isTask
     * @param null $usedInTask
     * @param null $usedInIssue
     */
	public function createTaskStatus($control,$label,$isIssue,$taskTypeId,$isInitial,$isTask = null,$usedInTask = null,$usedInIssue = null)
	{
		if(!empty($taskTypeId)){
            $taskType = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR.'TaskType')->findBy(array('id' => $taskTypeId))[0];
		}else{
            $taskType = null;
        }
		$taskStatusEntity = $this->taskOptionsFactory->createStatus($label,$isIssue,$control,$taskType,$isTask,$usedInTask,$usedInIssue);
		$this->entityManager->persist($taskStatusEntity);
		$this->updateTaskTypeFromTaskStatus($taskStatusEntity,$taskType ,$isInitial);
		$this->entityManager->flush();
		
		return array("ID" => $taskStatusEntity->getId());
	}

    /**
     * Updates an existent task status.
     * @param $idTaskStatus
     * @param $control
     * @param $label
     * @param $isIssue
     * @param $taskTypeId
     * @param $isInitial
     * @param null $isTask
     * @param null $usedInTask
     * @param null $usedInIssue
     */
	public function updateTaskStatus($idTaskStatus,$control,$label,$isIssue,$taskTypeId,$isInitial,$isTask = null,$usedInTask = null,$usedInIssue = null){

		# If TASK_TYPE has any value, gets a TASK_TYPE entity
		if(!empty($taskTypeId)){
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskType $taskType */
            $taskType = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR.'TaskType')->findOneBy(array('id' => $taskTypeId));
		} else {
            $taskType = null;
			$taskTypeEntityToClear = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR.'TaskType')->findOneBy(array('initialTaskStatus' => $idTaskStatus));
            if(!empty($taskTypeEntityToClear)){

                $taskTypeEntityToClear = $this->taskOptionsFactory->updateTaskTypeFromTaskStatus($taskTypeEntityToClear, null);
                $this->entityManager->persist($taskTypeEntityToClear);
            }
        }

        # Gets TASK_STATUS entity
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskStatus $taskStatusEntity */
		$taskStatusEntity = $this->entityManager->getRepository(Utilities::REPOSITORY_DIR.'TaskStatus')->findOneBy(array('id' => $idTaskStatus));
        # Save on TASK_STATUS
        $taskStatusEntity = $this->taskOptionsFactory->updateStatus($taskStatusEntity, $label,$isIssue,$control,$taskType,$isTask,$usedInTask,$usedInIssue);

        $this->entityManager->persist($taskStatusEntity);

		$this->updateTaskTypeFromTaskStatus($taskStatusEntity, $taskType ,$isInitial);

		$this->entityManager->flush();
	}

    /**
     * Checks if IS_INITIAL checkbox was checked and do the right
     * things. Since updateTaskStatus() and createTaskStatus() use
     * these same steps, we modularized this method.
     *
     * @param $taskStatusEntity
     * @param $taskType
     * @param $isInitial
     */
	private function updateTaskTypeFromTaskStatus($taskStatusEntity = null,$taskType,$isInitial){
		$taskTypeEntity = null;
		if(!empty($taskType)){
			if($isInitial == 'Y'){
				$taskTypeEntity = $this->taskOptionsFactory->updateTaskTypeFromTaskStatus($taskType, $taskStatusEntity);
			} else {
				$taskTypeEntity = $this->taskOptionsFactory->updateTaskTypeFromTaskStatus($taskType);
			}
			$this->entityManager->persist($taskTypeEntity);
		}
	}

}